
# -*- coding: utf-8 -*-
"""
Created on Fri Feb 18 11:05:22 2022

@author: pwiechen
"""

#%%
import numpy as np   
import pandas as pd       
import matplotlib.pyplot as plt     
import os
from scipy.interpolate import griddata, interp1d
import matplotlib.path as mpltPath

dir_path = os.path.dirname(os.path.abspath(__file__))
os.chdir(dir_path)

print(dir_path)

#%%

def cross_section_calculator(x_profile, y_profile, gps_file, total_width_dune_territory):
    '''
    Function to project all GPS datapoints from the gps_file total_width_dune_territory/2 left and right from the central cross section
    on the central cross section, and then interpolate linearly that data to acquire the cross sectional profile along x_profile
    and y_profile.     
    
    input:
    x_profile: x-coordinates of the desired central cross section
    y_profile: x-coordinates of the desired central cross section
    gps_file: dataframe with the GPS coordinates
    total_width_dune_territory: this length from left to right from the central cross section marks the boundary from where points will be projected. 

    returns:
    z_profile: z_values at coordinates x_profile and y_profile based on the interpolation of the projected GPS coordinates on the central array
    '''

    # Determine the points that will be projected

    orientation_dune = np.arctan((y_profile[-1] - y_profile[0])/(x_profile[-1] - x_profile[0])) + np.pi
    pol_dune = np.array([[x_profile[0]  - total_width_dune_territory/2*np.sin(np.pi - orientation_dune), y_profile[0]  - total_width_dune_territory/2*np.cos(np.pi - orientation_dune)],
                         [x_profile[0]  + total_width_dune_territory/2*np.sin(np.pi - orientation_dune), y_profile[0]  + total_width_dune_territory/2*np.cos(np.pi - orientation_dune)],
                         [x_profile[-1] + total_width_dune_territory/2*np.sin(np.pi - orientation_dune), y_profile[-1] + total_width_dune_territory/2*np.cos(np.pi - orientation_dune)],
                         [x_profile[-1] - total_width_dune_territory/2*np.sin(np.pi - orientation_dune), y_profile[-1] - total_width_dune_territory/2*np.cos(np.pi - orientation_dune)],
                         [x_profile[0]  - total_width_dune_territory/2*np.sin(np.pi - orientation_dune), y_profile[0]  - total_width_dune_territory/2*np.cos(np.pi - orientation_dune)]])
    
    path_dune = mpltPath.Path(pol_dune)
    points_gps = np.vstack((gps_file['x_val'],gps_file['y_val'])).T 
    grid = path_dune.contains_points(points_gps)
    gps_filtered = gps_file[grid]
    
    alpha_cross = orientation_dune - np.pi
    
    # Compute projections

    projection_x = np.zeros(len(gps_filtered))
    projection_y = np.zeros(len(gps_filtered))
    
    for i in range(len(gps_filtered)):

        beta_point = np.arctan((gps_filtered['y_val'].values[i] - y_profile[0])/(gps_filtered['x_val'].values[i] - x_profile[0]))
        dist_gpspoint_to_origin = np.sqrt((gps_filtered['x_val'].values[i] - x_profile[0])**2 + (gps_filtered['y_val'].values[i] - y_profile[0])**2)
        dist_gpspoint_to_cross_section = np.sin(beta_point - alpha_cross)*dist_gpspoint_to_origin
        
        dx = np.sin(alpha_cross)*dist_gpspoint_to_cross_section
        dy = np.cos(alpha_cross)*dist_gpspoint_to_cross_section
        
        projection_x[i] = gps_filtered['x_val'].values[i] + dx
        projection_y[i] = gps_filtered['y_val'].values[i] - dy
    
    # Interpolate the projected points to the central cross section

    interpolation_function = interp1d(np.sqrt((projection_x-x_profile[0])**2 + (projection_y-y_profile[0])**2),
                                      gps_filtered['z_val'], assume_sorted=False, bounds_error=False, fill_value=np.nan, kind='slinear')
    z_profile = interpolation_function(np.sqrt((x_profile-x_profile[0])**2 + (y_profile-y_profile[0])**2))
    z_profile_griddata = griddata((gps_file['x_val'],gps_file['y_val']), gps_file['z_val'],(x_profile, y_profile), method='linear')

    # Check how well the projection works

    fig, (ax1, ax2) = plt.subplots(2, figsize=(8,6))
    
    ax1.scatter(gps_file['x_val'], gps_file['y_val'], c = gps_file['z_val'])
    ax1.scatter(gps_filtered['x_val'], gps_filtered['y_val'], color='red', marker='x')
    ax1.plot(x_profile, y_profile)
    ax1.scatter(projection_x, projection_y, color='C1')
               
    ax2.scatter(projection_x, gps_filtered['z_val'], label='Projected values', color='C3')
    ax2.plot(x_profile, z_profile, label='Interpolated values', color='black')
    ax2.plot(x_profile, z_profile_griddata, label='Griddata', color='blue')
    
    ax2.legend()
    
    return z_profile

#%% Dune coordinates

# Dune 1 coordinates

x_extension_seaward = 450
x_extension_landward = 50
amount_of_grid_points = 10*(x_extension_seaward +  x_extension_landward)

origin_dune_1 = (72314.8, 451899)
orientation_dune_1 = 2*np.pi + -(-np.pi/2 + 296/180*np.pi)

x_cross_dune_1 = np.linspace(origin_dune_1[0]+x_extension_seaward*np.cos(orientation_dune_1), origin_dune_1[0]-x_extension_landward*np.cos(orientation_dune_1), amount_of_grid_points)
y_cross_dune_1 = origin_dune_1[1] + np.tan(orientation_dune_1)*(x_cross_dune_1 - origin_dune_1[0])
cross_1d_dune_1 = np.linspace(-x_extension_seaward, x_extension_landward, len(x_cross_dune_1))

# Dune 2 coordinates

origin_dune_2 = (72587.6, 452348)
orientation_dune_2 = 2*np.pi + -(-np.pi/2 + 310/180*np.pi)

x_cross_dune_2 = np.linspace(origin_dune_2[0]+x_extension_seaward*np.cos(orientation_dune_2), origin_dune_2[0]-x_extension_landward*np.cos(orientation_dune_2), amount_of_grid_points)
y_cross_dune_2 = origin_dune_2[1] + np.tan(orientation_dune_2)*(x_cross_dune_2 - origin_dune_2[0])
cross_1d_dune_2 = np.linspace(-x_extension_seaward, x_extension_landward, len(x_cross_dune_2))


#%%

types_dict = {'description': str,'date':str, 'time_raw':str, 'x_val': float, 'y_val': float, 'z_val': float, '2d_error': float, '3d_error': float}
max_error = 0.05 

# Compute projected cross sections of November 24
        
gps_20211124 = pd.read_csv(r'../../GPS/20211124/20211124_TRANSECTS.txt', usecols=[0,1,2,3,4,5,6,7], index_col=False,delim_whitespace=True, skipinitialspace=True, names=['description','date', 'time_raw', 'x_val', 'y_val', 'z_val', '2d_error', '3d_error'], dtype=types_dict, parse_dates={'time':['date', 'time_raw']})
gps_20211124 = gps_20211124.drop(gps_20211124[gps_20211124['3d_error'] > max_error].index)

cross_section_dune_1 = gps_20211124[gps_20211124.description.str.contains('TRANSECT_DUNE_1')]
cross_section_dune_2 = gps_20211124[gps_20211124.description.str.contains('TRANSECT_DUNE_2')]

z_proj_20211124_dune_1 = cross_section_calculator(x_cross_dune_1, y_cross_dune_1, cross_section_dune_1, 2*3.5)
z_proj_20211124_dune_2 = cross_section_calculator(x_cross_dune_2, y_cross_dune_2, cross_section_dune_2, 2*3.5)

# Extract x and y coordinates of S01_RBR_01 and S11_RBR_04 based on the poles to which the instruments were attached

gps_20211206 = pd.read_csv(r'../../GPS/20211206/20211206_INSTRUMENTS.txt', usecols=[0,1,2,3,4,5,6,7], index_col=False,delim_whitespace=True, skipinitialspace=True, names=['description','date', 'time_raw', 'x_val', 'y_val', 'z_val', '2d_error', '3d_error'], dtype=types_dict, parse_dates={'time':['date', 'time_raw']})
gps_20211206 = gps_20211206.drop(gps_20211206[gps_20211206['3d_error'] > max_error].index)
s01_20211206 = np.mean(gps_20211206[gps_20211206.description.str.contains('S01_POLE_TOP_01')].values[:,2:], axis=0)
s11_20211206 = np.mean(gps_20211206[gps_20211206.description.str.contains('S11_POLE_TOP_01')].values[:,2:], axis=0)

# Find projections of these coordinates on the central cross section

s01_angle = np.arctan((s01_20211206[1] - y_cross_dune_1[0])/(s01_20211206[0] - x_cross_dune_1[0]))
s01_radius = np.sqrt((s01_20211206[0] - x_cross_dune_1[0])**2 + (s01_20211206[1] - y_cross_dune_1[0])**2)
s01_dist_gpspoint_to_cross_section = np.sin(s01_angle - np.arctan((y_cross_dune_1[-1] - y_cross_dune_1[0])/(x_cross_dune_1[-1] - x_cross_dune_1[0])))*s01_radius
s01_projection_x =  s01_20211206[0] + np.sin(np.arctan((y_cross_dune_1[-1] - y_cross_dune_1[0])/(x_cross_dune_1[-1] - x_cross_dune_1[0])))*s01_dist_gpspoint_to_cross_section
s01_projection_y =  s01_20211206[1] - np.cos(np.arctan((y_cross_dune_1[-1] - y_cross_dune_1[0])/(x_cross_dune_1[-1] - x_cross_dune_1[0])))*s01_dist_gpspoint_to_cross_section
s01_projection_1d = (origin_dune_1[0]-s01_projection_x)/np.cos(orientation_dune_1)

s11_angle = np.arctan((s11_20211206[1] - y_cross_dune_2[0])/(s11_20211206[0] - x_cross_dune_2[0]))
s11_radius = np.sqrt((s11_20211206[0] - x_cross_dune_2[0])**2 + (s11_20211206[1] - y_cross_dune_2[0])**2)
s11_dist_gpspoint_to_cross_section = np.sin(s11_angle - np.arctan((y_cross_dune_2[-1] - y_cross_dune_2[0])/(x_cross_dune_2[-1] - x_cross_dune_2[0])))*s11_radius
s11_projection_x =  s11_20211206[0] + np.sin(np.arctan((y_cross_dune_2[-1] - y_cross_dune_2[0])/(x_cross_dune_2[-1] - x_cross_dune_2[0])))*s11_dist_gpspoint_to_cross_section
s11_projection_y =  s11_20211206[1] - np.cos(np.arctan((y_cross_dune_2[-1] - y_cross_dune_2[0])/(x_cross_dune_2[-1] - x_cross_dune_2[0])))*s11_dist_gpspoint_to_cross_section
s11_projection_1d = (origin_dune_2[0] - s11_projection_x)/np.cos(orientation_dune_2)

s01_h_bed_20211124 = 0.96
s11_h_bed_20211124 = 0.68

#%%

instrument_height_rbr = 0.215

print('x on November 24th S01_RBR_01:', np.round(s01_20211206[0],4))
print('y on November 24th S01_RBR_01:', np.round(s01_20211206[1],4))
print('z_top on November 24th S01_RBR_01:', np.round(s01_h_bed_20211124 + interp1d(x_cross_dune_1, z_proj_20211124_dune_1, kind='linear')(s01_projection_x) + instrument_height_rbr,4))

print('x on November 24th S11_RBR_04:', np.round(s11_20211206[0],4))
print('y on November 24th S11_RBR_04:', np.round(s11_20211206[1],4))
print('z_top on November 24th S11_RBR_04:', np.round(s11_h_bed_20211124 + interp1d(x_cross_dune_2, z_proj_20211124_dune_2, kind='linear')(s11_projection_x) + instrument_height_rbr,4))


#%%
fig, (ax1, ax2) = plt.subplots(nrows=2, sharex=True, sharey=True)

ax1.plot(cross_1d_dune_1, z_proj_20211124_dune_1, label='Survey 2021-11-24')
ax1.scatter(s01_projection_1d,interp1d(x_cross_dune_1, z_proj_20211124_dune_1, kind='linear')(s01_projection_x), label='interpolation S01')
ax1.scatter(s01_projection_1d,interp1d(x_cross_dune_1, z_proj_20211124_dune_1, kind='linear')(s01_projection_x) + s01_h_bed_20211124 + instrument_height_rbr, label='Estimation S01_RBR_01')

ax2.plot(cross_1d_dune_2, z_proj_20211124_dune_2, label='Survey 2021-11-24')
ax2.scatter(s11_projection_1d,interp1d(x_cross_dune_2, z_proj_20211124_dune_2, kind='linear')(s11_projection_x), label='interpolation S11')
ax2.scatter(s11_projection_1d,interp1d(x_cross_dune_2, z_proj_20211124_dune_2, kind='linear')(s11_projection_x) + s11_h_bed_20211124 + instrument_height_rbr, label='Estimation S11_RBR_04')

ax1.legend()
ax2.legend()
ax1.grid()
ax2.grid()
ax1.set(ylabel='Elevation [m NAP]')
ax2.set(xlabel='Cross-shore coordinate [m]', ylabel='Elevation [m NAP]')

#%%